#!/usr/bin/python
import reclasso

import numpy as np
from time import time as now
import sys, getopt
from decimal import *
from array import *
import random
from reclasso import algo
import pickle
import os

def compare_algos(X,y,n=250, m=100, d=25, N=10, mu=.1, sigma=0.01, verbose=True, showfig=False, simuname='test', first_time=True):
    """
    batch experiment
    compare performance of reclasso, lars, and coordinate descent with warm start
    """
    
    # sample observations
    thetaopt = np.zeros((m, 1))
    thetaopt[:d] = 1.
    err_hamming = np.empty((n-1, N))    # hamming distance between the current solution and hte optimal solution
    err_l2 = np.empty((n-1, N))         # l2 distance between the current solution and hte optimal solution
    
    # regularization parameter schedule
    lam = mu * np.arange(1, n+1)
    
    for j in range(1):
        
        # sample data
        #X = np.random.randn(n, m)
        #y = np.dot(X, thetaopt) + sigma*np.sqrt(d)*np.random.randn(n,1)
        
        if first_time:
            # get first solution
            theta_nz, nz, K = algo.one_observation(X[0,:], y[0], lam[0])
            theta = np.zeros((m, 1))
            theta[nz] = theta_nz
        
            for i in range(1, n):
            
                if verbose: print '\nsimulation %d, observation %d'%(j, i)
               
                
                # solve recursive lasso
                theta_nz, nz, K, nbr1, nbr2 = reclasso.reclasso(X[:i+1, :], y[:i+1], lam[i-1], lam[i], theta_nz, nz, K, verbose=verbose)

            
                # update current solution using the homotopy solution
                theta = np.zeros((m, 1))
                theta[nz] = theta_nz
                
                #err_l2[i-1, j] = np.sqrt(((theta - thetaopt)**2).sum()) / np.sqrt((thetaopt**2).sum())
                #err_hamming[i-1, j] = len(np.where((abs(thetaopt) > 0) != (abs(theta) > 0))[0])

            with open('objs.pickle', 'w') as f:
                pickle.dump([theta_nz, nz, K], f)
            # look at the error terms
            #err_l2[i-1, j] = np.sqrt(((theta - thetaopt)**2).sum()) / np.sqrt((thetaopt**2).sum())
            #err_hamming[i-1, j] = len(np.where((abs(thetaopt) > 0) != (abs(theta) > 0))[0])
        else:
            # not first time, load the already built model
            with open('objs.pickle') as f:
                theta_nz, nz, K = pickle.load(f)
                for i in range(1, n):
            
                    if verbose: print '\nsimulation %d, observation %d'%(j, i)

                    theta_nz, nz, K, nbr1, nbr2 = reclasso.reclasso(X[:i+1, :], y[:i+1], lam[i-1], lam[i], theta_nz, nz, K, verbose=verbose)
            
                    # update current solution using the homotopy solution
                    theta = np.zeros((m, 1))
                    theta[nz] = theta_nz
            os.remove('objs.pickle')
            with open('objs.pickle', 'w') as f:
                pickle.dump([theta_nz, nz, K], f)
    print 'Theta ', theta.T
    #print 'Error ', err_l2
    #print 'Errorh ', err_hamming
        


Y = np.zeros((20,1))
predictors = 0
datapoints = 0

with open("profile.txt") as f:
    for line in f:
        line_content1 = line.split()
        predictors = len(line_content1)-1
        break
    X = np.zeros((20,predictors))		
    for line in f:
        #print line
        line_content = line.split()
        #print predictors
        Y[datapoints,0] = float(line_content[0])
        for i in range(0,predictors):
            X[datapoints,i] = line_content[i+1]
        datapoints = datapoints + 1
d = predictors/4
        
compare_algos(X,Y,datapoints,predictors, d, 20, .1, 0.01, False, False, 'test', True)
#compare_algos(X,Y,datapoints,predictors, d, 10, .1, 0.01, False, False, 'test', False)
